:- module(auxiliaries, [plot_imp/2, check_sat_clauses/1, format_vars/1]).

:- use_module(library(lists)).
:- use_module(library(assoc)).
:- use_module(library(between)).

%------------------------------------------------------------------------------
% Below here are just auxilliary functions for output, debugging, plotting etc
%------------------------------------------------------------------------------

plot_imp(Why, File) :-
	open(File, write, Stream),
	format(Stream, "strict digraph graphname {~nrankdir=LR;~nnode [shape=circle color=white];~nkappakappa [label=\"&kappa;\"];~n", []),
	plot_imp(Why, ('kappa', 'kappa'), Stream),
	format(Stream, "}~n", []),
	close(Stream).

plot_imp([],_,_).
plot_imp([imp(Level, Id, Pol, [])| Whys], To, Stream) :-
	!,
	format(Stream, "~t~w~w [color=black shape=ellipse label=\"~w=~w@~w\"];~n", [Pol, Id, Id, Pol, Level]),
	To = (ToId, ToPol),
	format(Stream, "~t~w~w -> ~w~w;~n", [Pol, Id, ToPol, ToId]),
	plot_imp(Whys, To, Stream).
plot_imp([imp(Level, Id, Pol, SubWhys) | Whys], To, Stream) :-
	format(Stream, "~t~w~w [label=\"~w=~w@~w\"];~n", [Pol, Id, Id, Pol, Level]),
	To = (ToId, ToPol),
	format(Stream, "~t~w~w -> ~w~w;~n", [Pol, Id, ToPol, ToId]),
	plot_imp(SubWhys, (Id, Pol), Stream),
	plot_imp(Whys, To, Stream).

check_sat_clauses([]).
check_sat_clauses([Clause | Clauses]) :-
	check_sat_clause(Clause),
	check_sat_clauses(Clauses).

check_sat_clause([Pol-(Var:_Why) | _Clause]) :-
	Pol == Var, !.
check_sat_clause([_ | Clause]) :-
	check_sat_clause(Clause).

format_vars(Vars) :-
	format("vars     : ~n", []),
	format("[ ", []),
	format_vars_aux(Vars),
	format("]~n", []).

format_vars_aux([]).
format_vars_aux([_:imp(_, Id, Pol, _) | Vars]) :-
	format(" ~w-~w", [Pol, Id]),
	format_vars_aux(Vars).
